@@ -27,7 +27,6 @@ require Exporter;
 
 use Mail::DKIM::Signer;
 use Mail::DKIM::Verifier;
-use Mail::DKIM::TextWrap;
 
 use Mail::MIMEDefang;
 
@@ -71,7 +70,7 @@ The algorithm to be used to sign the message, by default is 'rsa-sha1'
 
 =item C<$method>
 
-The method used to sign the message, by default is 'relaxed'
+The method used to sign the message, by default is 'relaxed/simple'
 
 =item C<$domain>
 
@@ -93,19 +92,31 @@ The headers to sign, by default the headers are:
                List-Id List-Help List-Unsubscribe List-Subscribe
                List-Post List-Owner List-Archive
 
+=item C<$wrap>
+
+Option to enable or disable DKIM header lines wrap (enabled by default).
+
 =back
 
 =cut
 
 sub md_dkim_sign {
 
-  my ($keyfile, $algorithm, $method, $domain, $selector, $headers) = @_;
+  my ($keyfile, $algorithm, $method, $domain, $selector, $headers, $wrap) = @_;
 
   $algorithm = defined $algorithm ? $algorithm : 'rsa-sha1';
-  $method = defined $method ? $method : 'relaxed';
+  $method = defined $method ? $method : 'relaxed/simple';
   $selector = defined $selector ? $selector : 'default';
+  $wrap //= 1;
 
-  my ($fh, $h, $v);
+  eval {
+    if($wrap) {
+      require Mail::DKIM::TextWrap;
+      Mail::DKIM::TextWrap->import();
+    }
+  };
+
+  my ($fh);
 
   if(not -f $keyfile) {
     md_syslog('err', "Could not open private DKIM key in md_dkim_sign: $!");
@@ -129,8 +140,7 @@ sub md_dkim_sign {
   # or read an email and pass it into the signer, one line at a time
   while (<$fh>) {
     # remove local line terminators
-    chomp;
-    s/\015$//;
+    s/\015?\012$//;
 
     # use SMTP line terminators
     $dkim->PRINT("$_\015\012");
@@ -138,26 +148,35 @@ sub md_dkim_sign {
   $dkim->CLOSE;
   close($fh);
 
-  my $signature = $dkim->signature;
-  if($signature->as_string =~ /^(.*):\s(.*)$/s) {
-    $h = $1;
-    $v = $2;
-    $v =~ s/\r//gs;
-    return ($h, $v);
+  my $signature = $dkim->signature->as_string;
+  # canonicalize newlines and trim trailing newline
+  $signature =~ s/\015(?=\012)//gs;
+  $signature =~ s/\012$//;
+
+  if($signature =~ /^(.*):\s(.*)$/s) {
+    return ($1, $2);
   }
   return;
 }
 
 =item md_dkim_verify
 
-Verifies the DKIM signature of an email.
-Return value can be "pass", "fail", "invalid", "temperror", "none".
+Verifies the DKIM signature of an email, this method has no parameters.
+
+The first return value can be "pass", "fail", "invalid", "temperror" or "none".
+
 In case of multiple signatures, the "best" result will be returned.
+
 Best is defined as "pass", followed by "fail", "invalid", and "none".
+
 The second return value is the domain that has applied the signature.
+
 The third return value is the size of the DKIM public key.
+
 The forth return value is the value of the "b" tag of the DKIM signature.
 
+=back
+
 =cut
 
 sub md_dkim_verify {
@@ -181,8 +200,7 @@ sub md_dkim_verify {
     };
     while (<$fh>) {
       # remove local line terminators
-      chomp;
-      s/\015$//;
+      s/\015?\012$//;
 
       # use SMTP line terminators
       $dkim->PRINT("$_\015\012");
@@ -202,8 +220,4 @@ sub md_dkim_verify {
   }
 }
 
-=back
-
-=cut
-
 1;
